<?php

namespace Drupal\config_purge;

use Drupal\Core\Config\InstallStorage;

trait ConfigPurgeTrait {

  /**
   * Get configs defined in a given module.
   *
   * @param string $module
   *   The module name.
   * @param string $config_type
   *   The type of the config file: 'base' or 'optional'.
   *
   * @return array
   *   The list of config files.
   */
  public function getConfigFiles(string $module, string $config_type = 'all'): array {
    $config_dir = \Drupal::service('extension.list.module')->getPath($module) . '/';

    switch ($config_type) {
      case 'base':
        $config_dir .= InstallStorage::CONFIG_INSTALL_DIRECTORY;
        break;

      case 'optional':
        $config_dir .= InstallStorage::CONFIG_OPTIONAL_DIRECTORY;
        break;

      case 'all':
        $configs['base'] = $this->getConfigFiles($module, 'base');
        $configs['optional'] = $this->getConfigFiles($module, 'optional');
        return array_merge($configs['base'], $configs['optional']);

      default:
        return [];
    }

    if (!file_exists($config_dir)) {
      return [];
    }

    return \Drupal::service('file_system')->scanDirectory($config_dir, "/\.(yml)$/");
  }

  /**
   * Get a list of active configurations related to given module.
   *
   * @param string $module
   *   The module name.
   *
   * @return array
   *   List of active configurations.
   */
  public function getActiveConfigurations(string $module): array {
    $config_files = $this->getConfigFiles($module);
    if (empty($config_files)) {
      return [];
    }

    $configs = [];
    $activeConfigFactory = \Drupal::service('config.factory');
    foreach ($config_files as $config_file) {
      $config_name = $config_file->name;
      if (!$activeConfigFactory->get($config_name)->isNew()) {
        $configs[$config_name] = $config_name;
      }
    }

    return $configs;
  }

  /**
   * Check if given module has active config.
   *
   * @param string $module
   *   The module name.
   *
   * @return bool
   *   Returns TRUE if there is active configuration.
   */
  public function hasActiveConfiguration(string $module): bool {
    return !empty($this->getActiveConfigurations($module));
  }

}
