<?php

namespace Drupal\config_purge\Form;

use Drupal\config_purge\ConfigPurgeTrait;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Builds a confirmation form to uninstall selected modules.
 */
class PurgeConfigurationsConfirmForm extends ConfirmFormBase {

  use ConfigPurgeTrait;

  public function __construct(
    private readonly KeyValueStoreExpirableInterface $keyValueExpirable,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('keyvalue.expirable')->get('config_purge')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Confirm purge configurations');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText(): TranslatableMarkup {
    return $this->t('Purge configurations');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('config_purge.purge');
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): TranslatableMarkup {
    return $this->t('Would you like to purge the selected configurations?');
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'purge_configuration_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): RedirectResponse|array {
    // Retrieve the list of modules from the key value store.
    $account = $this->currentUser()->id();
    $modules = $this->keyValueExpirable->get($account);
    if (empty($modules)) {
      return $this->redirect('config_purge.purge');
    }

    foreach ($modules as $module => $module_name) {
      $config_files['base'] = $this->getConfigFiles($module, 'base');
      $config_files['optional'] = $this->getConfigFiles($module, 'optional');

      if (empty($config_files['base']) && empty($config_files['optional'])) {
        // Module without config.
        continue;
      }

      $active_configs = $this->getActiveConfigurations($module);
      $form['config_purge'][$module] = [
        '#type' => 'details',
        '#title' => $module_name . ' (' . $module . ')',
        '#description' => t('Select the configurations to be deleted.'),
        '#weight' => 0,
        '#validated' => TRUE,
        '#open' => TRUE,
      ];

      $config_lists = [
        'base' => t('Configurations'),
        'optional' => t('Optional configurations'),
      ];

      foreach ($config_lists as $type => $title) {
        if (empty($config_files[$type])) {
          continue;
        }

        $configs = [];
        foreach ($config_files[$type] as $config_file) {
          $config_name = $config_file->name;
          $configs[$config_name] = $config_name;
          if (in_array($config_name, $active_configs)) {
            $configs[$config_name] .= ' ' . $this->t('(active)');
          }
        }

        if (!empty($configs)) {
          asort($configs);
          $form['config_purge'][$module][$type . '_configs'] = [
            '#type' => 'checkboxes',
            '#title' => $title,
            '#options' => $configs,
            '#validated' => TRUE,
          ];
        }
      }
    }

    $form['purge_all_configs'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Delete all the listed configurations.'),
      '#validated' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    foreach (['base', 'optional'] as $type) {
      $configs_selected[$type] = $form_state->getValue($type . '_configs');
      if (empty($configs_selected[$type])) {
        $configs_selected[$type] = [];
      }
    }
    $configs = array_merge($configs_selected['base'], $configs_selected['optional']);
    $purge_all = !empty($form_state->getValue('purge_all_configs'));

    foreach ($configs as $config_name => $checked) {
      if ($purge_all || $checked) {
        $this->configFactory()->getEditable($config_name)->delete();
      }
    }

    // Delete the keyValue of current user.
    $account = $this->currentUser()->id();
    $this->keyValueExpirable->delete($account);

    $this->messenger()->addMessage($this->t('The selected configurations have been deleted.'));
    $form_state->setRedirectUrl($this->getCancelUrl());
  }

}
