<?php

namespace Drupal\config_purge\Form;

use Drupal\config_purge\ConfigPurgeTrait;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\InfoParserException;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * List all uninstalled module with active config.
 */
class PurgeConfigurationsForm extends FormBase {

  use ConfigPurgeTrait;

  public function __construct(
    private readonly ModuleHandlerInterface $moduleHandler,
    private readonly KeyValueStoreExpirableInterface $keyValueExpirable,
    private readonly AccountInterface $currentUser,
    private readonly ModuleExtensionList $moduleExtensionList,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): PurgeConfigurationsForm|static {
    return new static(
      $container->get('module_handler'),
      $container->get('keyvalue.expirable')->get('config_purge'),
      $container->get('current_user'),
      $container->get('extension.list.module')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'config_purge_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    require_once DRUPAL_ROOT . '/core/includes/install.inc';

    // Include system.admin.inc so we can use the sort callbacks.
    $this->moduleHandler->loadInclude('system', 'inc', 'system.admin');

    // Sort all modules by their names.
    try {
      // The module list needs to be reset so that it can re-scan and include
      // any new modules that may have been added directly into the filesystem.
      $modules = $this->moduleExtensionList->reset()->getList();
      uasort($modules, [ModuleExtensionList::class, 'sortByName']);
    }
    catch (InfoParserException $e) {
      $this->messenger()
        ->addError($this->t('Modules could not be listed due to an error: %error', ['%error' => $e->getMessage()]));
      $modules = [];
    }

    // Iterate over each of the modules.
    $form['modules']['#tree'] = TRUE;
    foreach ($modules as $filename => $module) {
      if (!empty($module->info['hidden'])) {
        continue;
      }
      if ($module->status) {
        continue;
      }
      if (!$this->hasActiveConfiguration($module->getName())) {
        continue;
      }

      $package = $module->info['package'];
      $form['modules'][$package][$filename] = $this->buildRow($module);
      $form['modules'][$package][$filename]['#parents'] = [
        'modules',
        $filename,
      ];
    }

    $packages = Element::children($form['modules']);
    if (empty($packages)) {
      $form['empty']['#markup'] = t("There isn't any uninstalled module with active configuration.");
      return $form;
    }

    // Add a wrapper around every package.
    foreach ($packages as $package) {
      $form['modules'][$package] += [
        '#type' => 'details',
        '#title' => $this->t("@package", ['@package' => $package]),
        '#open' => TRUE,
        '#theme' => 'config_purge_modules_details',
        // Ensure that the "Core" package comes first.
        '#weight' => $package == 'Core' ? -10 : NULL,
      ];
    }

    // Lastly, sort all packages by title.
    uasort($form['modules'], [
      '\Drupal\Component\Utility\SortArray',
      'sortByTitleProperty',
    ]);

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Purge'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * Builds a table row for the system modules page.
   *
   * @param \Drupal\Core\Extension\Extension $module
   *   The module for which to build the form row.
   *
   * @return array
   *   The form row for the given module.
   */
  protected function buildRow(Extension $module): array {
    $row['name']['#markup'] = $module->info['name'];
    $row['description']['#markup'] = $this->t('@desc', ['@desc' => $module->info['description']]);
    $row['version']['#markup'] = $module->info['version'];

    $row['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Purge'),
    ];

    return $row;
  }

  /**
   * Helper function for building a list of modules to purge.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   An array of modules to purge.
   */
  protected function buildModuleList(FormStateInterface $form_state): array {
    // Build a list of selected modules.
    $modules = [];

    $data = $this->moduleExtensionList->getList();
    foreach ($data as $name => $module) {
      $checkbox = $form_state->getValue(['modules', $name], FALSE);
      if (!empty($checkbox['enable'])) {
        $modules[$name] = $module->info['name'];
      }
    }

    return $modules;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $modules = $this->buildModuleList($form_state);

    // Write the list of changed module states into a key value store.
    $account = $this->currentUser()->id();
    $this->keyValueExpirable->setWithExpire($account, $modules, 60);

    // Redirect to the confirmation form.
    $form_state->setRedirect('config_purge.purge_confirm');
  }

}
